/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/o2kxml/. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2000-2002. All Rights Reserved. 

Contributor(s): 
*/

package com.simonstl.gorille;


import java.io.*;

/**

 * 
 * 
 * @author Simon St.Laurent
 * @version 0.01 $Date: 2002/10/17 $
 */
public final class BrutEOL extends FilterReader
{
   
    protected StringBuffer myBuffer = new StringBuffer();
    
    public BrutEOL (InputStream in) {
      this(new InputStreamReader(in));
    }

    public BrutEOL (Reader in) {
      super(new BufferedReader(in));
    }

/**
To keep the XML declaration, call keepXMLDecl(true) before processing the document.
**/


    protected int feedFromInternalBuffer() {
        //feed out the buffer a character at a time
        int retChar=myBuffer.charAt(0);
        myBuffer.reverse();
        myBuffer.setLength(myBuffer.length()-1);
        myBuffer.reverse();
        return retChar;
    }
    
    public int read() throws IOException {

      int c = ' '; //return a space if nothing else

      if (myBuffer.length()==0) {
       c=in.read();
       if (c==0xD) {
          int d=in.read();
          if ((d==0xA) || (d==0x85)) {
            myBuffer.append((char)0xA);
          } else {
            myBuffer.append((char)0xA);
            myBuffer.append((char) d );
          }
       }
       
       if ((c==0x85) || (c==0x2028)){
          myBuffer.append((char)0xA);
       }
       
      } //end buffer test

      if (myBuffer.length()>0) {
         c=feedFromInternalBuffer();
      } 
        return c;
    }

    /**
     * This method is a placeholder - all 'real' activity appears in 
     * the int read() method.  
     * This placeholder is substantially from Java I/O by 
     * Elliotte Rusty Harold, http://www.oreilly.com/catalog/javaio/.
     */
    private boolean endOfStream = false;
  
    public int read(char[] text, int offset, int length) throws IOException {
  
        if (endOfStream) return -1;
        int numRead = 0;
        
        for (int i = offset; i < offset+length; i++) {
            int temp = this.read();
            if (temp == -1) {
              this.endOfStream = true;
              break;
            }
            text[i] = (char) temp;
            numRead++;
        }
        return numRead;
  
    }

    /** This class defines a main() method to run this from the command line */
    public static void main(String[] args) {
        
        if (args.length != 1) {
            System.out.println("Usage: java BrutEOL <filename>");
            return;       
        }
        try {          
            // Create a stream to read and clean the file
            BrutEOL fixer=new BrutEOL(new FileReader(args[0]));
//            blogFixer.keepXMLDecl(true);
            BufferedReader in = new BufferedReader(fixer);
            String line;
            while((line = in.readLine()) != null) {
              System.out.println(line);
            }
            in.close();  // Close the stream.
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }
    
}
