package com.simonstl.gorille;



/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>This class stores a set of bounds for characters or a single acceptable value, making it easy to define rule sets separately from the vocabulary defined for the examples.</p>
*
*<p>Version 0.02 changes from char values to int values in order to support Unicode characters outside of Plane 0.</p>
*
* @version 0.02 9 January 2001
* @author Simon St.Laurent
*/

public class CharRange {

protected int start;
protected int end;
protected boolean singleton=false;

public CharRange(){
}

public CharRange(int character){
   start=character;
   singleton=true;
}

public CharRange(int _start, int _end){
   start=_start;
   end=_end;
}

public boolean isInRange(int test) {
  boolean returnVal=false;
  if (singleton) {
    if (test==start) {
      returnVal=true;
    } 
  } else {
    if ( (test>=start) && ( (test<=end)) ) {
      returnVal=true;
    } 
  }
  return returnVal;
}

public String toString(){
  String returnVal;

  if (singleton) {
    returnVal="Value: "+start;
  } else {
    returnVal="Start: "+start+" End: "+end;
  }
  return returnVal;
}

public void print(){
  System.out.println(this.toString());
}

public static void main (String[] args) {
  CharRange testRange=new CharRange('B', 'E');
  testRange.print();
  
  boolean result=testRange.isInRange('D');
  System.out.println("D is in the range above: "+result);
  result=testRange.isInRange('B');
  System.out.println("B is in the range above: "+result);
  result=testRange.isInRange('E');
  System.out.println("E is in the range above: "+result);
  result=testRange.isInRange('d');
  System.out.println("d is in the range above: "+result);
  result=testRange.isInRange('X');
  System.out.println("X is in the range above: "+result);

  System.out.println();

  testRange=new CharRange('D');
  testRange.print();
  result=testRange.isInRange('D');
  System.out.println("D is in the range above: "+result);
  result=testRange.isInRange('B');
  System.out.println("B is in the range above: "+result);
  result=testRange.isInRange('d');
  System.out.println("d is in the range above: "+result);
  result=testRange.isInRange('e');
  System.out.println("e is in the range above: "+result);

  System.out.println();


  testRange=new CharRange('B', 'b');
  testRange.print();
  result=testRange.isInRange('A');
  System.out.println("A is in the range above: "+result);
  result=testRange.isInRange('B');
  System.out.println("B is in the range above: "+result);
  result=testRange.isInRange('a');
  System.out.println("a is in the range above: "+result);
  result=testRange.isInRange('e');
  System.out.println("e is in the range above: "+result);


}

}