package com.simonstl.gorille;

import org.xml.sax.*;
import org.xml.sax.helpers.*;
import java.util.*;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 

Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
*
* <p>A SAX filter which tests information against the XML character rules.  This code has not been well-tested, as SAX parsers perform such testing in advance of the information reaching this filter.  This filter is intended as a demonstration of how such layering might work if parsers outsourced that work, and also as a tool for checking SAX events generated internally by programs before passing them to other processors.</p>
*
* @version 0.01  5 January 2002
* @author Simon St.Laurent
*/

public class CharRulesFilter extends XMLFilterImpl { 
    public static final int FATAL_ERROR=0;
    public static final int ERROR=1;
    public static final int WARNING=2;

    protected CharRules rules=null;   //set of rules
    protected int errorHandling=FATAL_ERROR;
    protected Locator locator=new LocatorImpl();
    protected boolean verbose=false;

/**
* An empty constructor that requires the use of setParent before starting
* I don't think this works.
*/
public CharRulesFilter() {

 } 

/**
* A constructor that takes the parser which will feed it SAX events
*/

public CharRulesFilter (XMLReader parent) {
    super(parent); 
} 

    /**
     * Set the value of a feature.
     *
     *
     * @param name The feature name.
     * @param value The requested feature value.
     * @exception org.xml.sax.SAXNotRecognizedException If the feature
     *            value can't be assigned or retrieved from the parent.
     * @exception org.xml.sax.SAXNotSupportedException When the
     *            parent recognizes the feature name but 
     *            cannot set the requested value.
     * @see org.xml.sax.XMLReader#setFeature
     */
    public void setFeature (String name, boolean value)
	throws SAXNotRecognizedException, SAXNotSupportedException
    {
//may be used to turn namespace-awareness on and off eventually
       super.setFeature (name, value);

    }


    /**
     * Look up the value of a feature.
     *
     * @param name The feature name.
     * @return The current value of the feature.
     * @exception org.xml.sax.SAXNotRecognizedException If the feature
     *            value can't be assigned or retrieved from the parent.
     * @exception org.xml.sax.SAXNotSupportedException When the
     *            parent recognizes the feature name but 
     *            cannot determine its value at this time.
     * @see org.xml.sax.XMLReader#getFeature
     */
    public boolean getFeature (String name)
	throws SAXNotRecognizedException, SAXNotSupportedException
    {
//will be used to report namespace-awareness
	return super.getFeature(name);
    }


    /**
     * Set the value of a property.
     *
     *
     * @param name The property name.
     * @param value The requested property value.
     * @exception org.xml.sax.SAXNotRecognizedException If the property
     *            value can't be assigned or retrieved from the parent.
     * @exception org.xml.sax.SAXNotSupportedException When the
     *            parent recognizes the property name but 
     *            cannot set the requested value.
     * @see org.xml.sax.XMLReader#setProperty
     */
    public void setProperty (String name, Object value)
throws SAXNotRecognizedException, SAXNotSupportedException {

    if (name.equals("com.simonstl.gorille.rules")) {
       this.setRules ( (CharRules) value); 
    } else {
      super.setProperty(name, value);
    }
 }


    /**
     * Look up the value of a property.
     *
     * @param name The property name.
     * @return The current value of the property.
     * @exception org.xml.sax.SAXNotRecognizedException If the property
     *            value can't be assigned or retrieved from the parent.
     * @exception org.xml.sax.SAXNotSupportedException When the
     *            parent recognizes the property name but 
     *            cannot determine its value at this time.
     * @see org.xml.sax.XMLReader#setFeature
     */
    public Object getProperty (String name)
	throws SAXNotRecognizedException, SAXNotSupportedException
    {    
    if (name.equals("com.simonstl.gorille.rules")) {
       return this.getRules (); 
    } else {
      return super.getProperty(name);
    }

    }


/**
* sets up the rules, read in from a config file by {@link com.simonstl.gorille.CharRulesLoader CharRulesLoader} or
* otherwise concocted in Java using the {@link com.simonstl.gorille.CharRules CharRules} class.
*
* @param newRules  the set of rules the class will apply to content
*/
public void setRules (CharRules newRules) {
  rules=newRules;
//rules.print();
}

/**
* Sets whether the filter should report errors to the command-line.  False by default.
**/

public void setVerbose (boolean _verbose) {
  verbose=_verbose;
}

/**
* sets up error-handling.  By default, character errors are fatal errors, but that may not always be appropriate.
*
* @param _ErrorHandling  the set of rules the class will apply to content
*/
public void setErrorType (int _errorHandling) {
  errorHandling=_errorHandling;
}


/**
* in case you ever need to get the rules out of the tester 
*/
public CharRules getRules () {
  return rules;
}

protected void reportError(SAXParseException e) {
  try {
    switch (errorHandling) {
      case WARNING:
if (verbose) {System.out.println("Warning: "+e);}
        this.warning(e);
      break;

      case ERROR:
if (verbose) {System.out.println("Error: "+e);}
        this.error(e);
      break;

      case FATAL_ERROR:
if (verbose) {System.out.println("Fatal Error: "+e);}
        this.fatalError(e);
      break;
    }
  } catch (SAXException error) {
     //do something
  }
}

//filters

/**
* Filter a new document locator event.
*
* Using setDocumentLocator is necessary if you want to know where an illegal character was encountered.
* @param locator The document locator.
*/
public void setDocumentLocator (Locator _locator) {
	this.locator = _locator;

      super.setDocumentLocator(_locator);
    }

    /**
     * Filter a notation declaration event.
     *
     * @param name The notation name.
     * @param publicId The notation's public identifier, or null.
     * @param systemId The notation's system identifier, or null.
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
    public void notationDecl (String name, String publicId, String systemId)
	throws SAXException
    {
     boolean succeeds=false;

    succeeds=rules.isName(name);

    if (!(succeeds)) {
          SAXParseException e=new SAXParseException("Notation name "+name+" is not a Name.",locator);
	    this.reportError(e);
    }

   succeeds=false;

    if (!(publicId==null)) {
      succeeds=rules.isPubid(publicId);

      if (!(succeeds)) {
            SAXParseException e=new SAXParseException("Notation PUBLIC Identifier "+publicId+" is not a Public ID.",locator);
	      this.reportError(e);
      }
    }

   succeeds=false;

    if (!(systemId==null)) {
      succeeds=rules.isChars(systemId);

      if (!(succeeds)) {
            SAXParseException e=new SAXParseException("Notation SYSTEM Identifier "+systemId+" contains illegal characters.",locator);
	      this.reportError(e);
      }
    }

    super.notationDecl (name, publicId, systemId);
    }

    
    /**
     * Filter an unparsed entity declaration event.
     *
     * @param name The entity name.
     * @param publicId The entity's public identifier, or null.
     * @param systemId The entity's system identifier, or null.
     * @param notationName The name of the associated notation.
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
public void unparsedEntityDecl (String name, String publicId,
				    String systemId, String notationName)
    throws SAXException {

     boolean succeeds=false;

    succeeds=rules.isName(name);

    if (!(succeeds)) {
          SAXParseException e=new SAXParseException("Unparsed Entity "+name+" is not a Name.",locator);
	    this.reportError(e);
    }

   succeeds=false;

    if (!(publicId==null)) {
      succeeds=rules.isPubid(publicId);

      if (!(succeeds)) {
            SAXParseException e=new SAXParseException("Unparsed Entity PUBLIC Identifier "+publicId+" is not a Public ID.",locator);
	      this.reportError(e);
      }
    }

   succeeds=false;

    if (!(systemId==null)) {
      succeeds=rules.isChars(systemId);

      if (!(succeeds)) {
            SAXParseException e=new SAXParseException("Unparsed Entity SYSTEM Identifier "+systemId+" contains illegal characters.",locator);
	      this.reportError(e);
      }
    }

   succeeds=false;

    if (!(notationName==null)) {
      succeeds=rules.isName(notationName);

      if (!(succeeds)) {
            SAXParseException e=new SAXParseException("Unparsed Entity Notation Identifier "+notationName+" is not a name.",locator);
	      this.reportError(e);
      }
    }

    super.unparsedEntityDecl (name, publicId, systemId, notationName);
    }



    ////////////////////////////////////////////////////////////////////
    // Implementation of org.xml.sax.ContentHandler.
    ////////////////////////////////////////////////////////////////////




    /**
     * Filter a start document event.
     *
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
    public void startDocument ()
	throws SAXException
    {

//nothing to check here, move along

    super.startDocument ();
    }


    /**
     * Filter an end document event.
     *
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
    public void endDocument ()
	throws SAXException
    {
//nothing to check here, move along

    super.endDocument ();
    }


    /**
     * Filter a start Namespace prefix mapping event.
     *
     * @param prefix The Namespace prefix.
     * @param uri The Namespace URI.
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
    public void startPrefixMapping (String prefix, String uri)
	throws SAXException
    {
//nothing to check here, move along

     super.startPrefixMapping (prefix, uri);
    }


    /**
     * Filter an end Namespace prefix mapping event.
     *
     * @param prefix The Namespace prefix.
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
    public void endPrefixMapping (String prefix)
	throws SAXException
    {
//nothing to check here, move along

    super.endPrefixMapping (prefix);
    }


    /**
     * Filter a start element event.
     *
     * @param uri The element's Namespace URI, or the empty string.
     * @param localName The element's local name, or the empty string.
     * @param qName The element's qualified (prefixed) name, or the empty
     *        string.
     * @param atts The element's attributes.
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
public void startElement (String uri, String localName, String qName,
			      Attributes atts)
throws SAXException {

     boolean succeeds=false;
     SAXParseException e;

    succeeds=rules.isQName(qName);

    if (!(succeeds)) {
          e=new SAXParseException("Element "+qName+" is not a QName.",locator);
	    this.reportError(e);
    }
    int numAtts=atts.getLength();

    for (int i=0; i<numAtts; i++) {
     String attQName=atts.getQName(i);

     succeeds=rules.isQName(attQName);

      if (!(succeeds)) {
          e=new SAXParseException("Attribute "+attQName+" is not a QName.",locator);
	    this.reportError(e);
      }//end if succeeds
      String attContent=atts.getValue(i);
      succeeds=rules.isChars(attContent);

    if (!(succeeds)) {
          e=new SAXParseException("Attribute "+attQName+" has faulty content: "+attContent+".",locator);
	    this.reportError(e);


    }//end if succeeds

    }//end for
    super.startElement (uri, localName, qName, atts);
    }


    /**
     * Filter an end element event.
     *
     * @param uri The element's Namespace URI, or the empty string.
     * @param localName The element's local name, or the empty string.
     * @param qName The element's qualified (prefixed) name, or the empty
     *        string.
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
public void endElement (String uri, String localName, String qName)
	throws SAXException
    {
     boolean succeeds=false;

    succeeds=rules.isQName(qName);

    if (!(succeeds)) {
          SAXParseException e=new SAXParseException("Element "+qName+" is not a QName.",locator);
	    this.reportError(e);
    }

  super.endElement (uri, localName, qName);
}


    /**
     * Filter a character data event.
     *
     * @param ch An array of characters.
     * @param start The starting position in the array.
     * @param length The number of characters to use from the array.
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
public void characters (char ch[], int start, int length) throws SAXException {  

    boolean succeeds=false;

    succeeds=rules.isChars(ch, start, length);

    if (!(succeeds)) {
      String value=new String(ch, start, length);

       SAXParseException e=new SAXParseException("Character content: \n "+value+"\n is not legitimate character content.",locator);
	    this.reportError(e);
    }

       super.characters (ch, start, length);
}


    /**
     * Filter an ignorable whitespace event.
     *
     * @param ch An array of characters.
     * @param start The starting position in the array.
     * @param length The number of characters to use from the array.
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
    public void ignorableWhitespace (char ch[], int start, int length)
	throws SAXException
    {
    boolean succeeds=false;

    succeeds=rules.isChars(ch, start, length);

    if (!(succeeds)) {
      String value=new String(ch, start, length);

       SAXParseException e=new SAXParseException("Ignorable whitespace: \n "+value+" \n is not legitimate character content.",locator);
	    this.reportError(e);
    }

    super.ignorableWhitespace (ch, start, length);
    }


    /**
     * Filter a processing instruction event.
     *
     * @param target The processing instruction target.
     * @param data The text following the target.
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
public void processingInstruction (String target, String data) throws SAXException {
    boolean succeeds=false;

    succeeds=rules.isName(target);

    if (!(succeeds)) {
          SAXParseException e=new SAXParseException("Processing Instruction target "+target+" is not a Name.",locator);
	    this.reportError(e);
    }

   succeeds=false;

    if (!(data==null)) {
      succeeds=rules.isChars(data);

      if (!(succeeds)) {
            SAXParseException e=new SAXParseException("Processing Instruction content: \n"+data+"\n for target "+target+"is not legitimate character content.",locator);
	      this.reportError(e);
      }
    }



      super.processingInstruction (target, data);
    }


    /**
     * Filter a skipped entity event.
     *
     * @param name The name of the skipped entity.
     * @exception org.xml.sax.SAXException The client may throw
     *            an exception during processing.
     */
public void skippedEntity (String name) throws SAXException {

    boolean succeeds=false;

    succeeds=rules.isName(name);

    if (!(succeeds)) {
          SAXParseException e=new SAXParseException("Skipped entity name "+name+" is not a Name.",locator);
	    this.reportError(e);
    }

    super.skippedEntity (name);

    }



} 