package com.simonstl.gorille;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import java.io.IOException;
import java.util.*;
import java.io.PrintStream;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001-2. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>This class writes Java code based on the contents of an XML Gorille config file.  The resulting class creates a derivative of {@link com.simonstl.gorille.CharRules CharRules} which begins with the rules specified by that file.</p>
<p>To use this class from the command-line, run the {@link com.simonstl.gorille.CharTester CharTester} class with the -g option.</p>
*
* <p>Version 0.02 shifts from attributes as content-holders to elements.</p>
*
* @version 0.02 16 October 2002
* @author Simon St.Laurent
*/

public class CharRulesGen extends DefaultHandler {
    
    public CharRules rules;   //set of rules
    protected CharRanges ranges;
    protected PrintStream output=System.out;
    protected String outPackage="com.simonstl.gorille";
    protected String outClass="GeneratedRules";

    protected String start=null;
    protected String end=null;
    protected StringBuffer collector=null;
    protected boolean startEnd=false;
    protected boolean listening=false;

    // Constructor
    public CharRulesGen (String xmlFile, XMLReader parser) {
        
        //  Set Content Handler
        parser.setContentHandler (this);

        // Create a rules container
        rules=new CharRules();

        // Create an initial range container
        ranges=new CharRanges();
 
        //  Parse the Document      
        try {
            parser.parse(xmlFile);
        } catch (SAXException e) {
            System.err.println (e);
        } catch (IOException e) {
            System.err.println (e);
        }
    }

public void setOutput(PrintStream _output) {
    output=_output;
}

public PrintStream getOutput(){
    return output;
}

public void setOutPackage (String _outPackage) {
    outPackage=_outPackage;
}

public String getOutPackage () {
    return outPackage;
}

public void setClass (String _outClass) {
    outClass=_outClass;
}

public String getOutClass () {
    return outClass;
}


public void startDocument() {
//write opening code
output.println("package "+outPackage+";");
output.println("import com.simonstl.gorille.*;");

output.println("/* The contents of this file are subject to the Mozilla Public License  Version 1.1 (the \"License\"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/  \n\nSoftware distributed under the License is distributed on an \"AS IS\"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. \n\nThe Original Code is available at  http://simonstl.com/projects/fragment/original.\n\nThe Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved.\n\n*/");
output.println("/**Generated code created by the com.simonstl.gorille.CharRulesGen class.**/");
output.println("public class "+outClass+" extends com.simonstl.gorille.CharRules{");
output.println("protected CharRanges ranges=new CharRanges();");

output.println("public "+outClass+" () {");
output.println(" CharRange range;");
}

public void endDocument() {
//write ending code

output.println("}//end constructor");
output.println("}//end class");


}


    //  Start Element Event Handler
public void startElement (String uri, String local,
        String qName, Attributes atts)  {

       
        if (uri.equals("http://simonstl.com/ns/gorille/")) {
//System.out.println("Starting: " + qName);
        if ( (local.equals("hex")) || (local.equals("chars")) ){
            start=null;
            end=null;
            collector=new StringBuffer();
            startEnd=false;
            listening=true;
        }

       if (local.equals("start")){
            startEnd=true;
       }



       }//end namespace
}//end startElement

public void characters(char[] ch, int offset, int len) throws SAXException{
//System.out.println(listening);
   if (listening) {
      collector.append(ch, offset, len);
   }
}

public void endElement (String uri, String local,
        String qName)  {
        if (uri.equals("http://simonstl.com/ns/gorille/")) {
        //System.out.println("Ending: " + qName);
          if (local.equals("start")){
            start=collector.toString();
            collector=new StringBuffer();
          }
          
          if (local.equals("end")){
            startEnd=true;
            end=collector.toString();
            collector=new StringBuffer();
          }
          
          if ( (local.equals("hex")) || (local.equals("chars")) ){
              if (startEnd==false) {
                 start=collector.toString();
              }
              startEnd=false;
              listening=false;
        }
           if (local.equals("hex")) {
             int startInt=Integer.decode(start).intValue();
//System.out.println("Starting at "+startInt);

             CharRange range;

             if (end==null) {
output.println("range=new CharRange("+startInt+");");
output.println("ranges.addRange(range);");
             } else {
             int endInt=Integer.decode(end).intValue();
//System.out.println("Ending at "+endInt);
output.println("range=new CharRange("+startInt+", "+endInt+");");
output.println("ranges.addRange(range);");
             }
           } //end hex
           
           
           if (local.equals("chars")) {
//System.out.println("Char is "+start);           
             int startInt=start.charAt(0);
//System.out.println("Char is "+startInt);

             if (end==null) {
output.println("range=new CharRange("+startInt+");");
output.println("ranges.addRange(range);");
             } else {
             int endInt=end.charAt(0);
;
//System.out.println("Char is "+endChar);
output.println("range=new CharRange("+startInt+", "+endInt+");");
output.println("ranges.addRange(range);");
             }
           } //end chars

        
        
           if (local.equals("Char")) {
             output.println("this.setChar(ranges);");
             output.println("");
             output.println("ranges=new CharRanges();");
           } //end Char
           if (local.equals("NameStartChar")) {
             output.println("this.setNameStartChar(ranges);");
             output.println("");
             output.println("ranges=new CharRanges();");
           } //end NameStartChar
           if (local.equals("AddlNameChar")) {
             output.println("this.setAddlNameChar(ranges);");
             output.println("");
             output.println("ranges=new CharRanges();");
           } //end AddlNameChar
           if (local.equals("Space")) {
             output.println("this.setSpace(ranges);");
             output.println("");
             output.println("ranges=new CharRanges();");
           } //end Space
           if (local.equals("BaseChar")) {
             output.println("this.setBaseChar(ranges);");
             output.println("");
             output.println("ranges=new CharRanges();");
           } //end BaseChar
           if (local.equals("Ideographic")) {
             output.println("this.setIdeographic(ranges);");
             output.println("");
             output.println("ranges=new CharRanges();");
           } //end Ideographic
           if (local.equals("CombiningChar")) {
             output.println("this.setCombiningChar(ranges);");
             output.println("");
             output.println("ranges=new CharRanges();");
           } //end CombiningChar
           if (local.equals("Digit")) {
             output.println("this.setDigit(ranges);");
             output.println("");
             output.println("ranges=new CharRanges();");
           } //end Digit
           if (local.equals("Extender")) {
             output.println("this.setExtender(ranges);");
             output.println("");
             output.println("ranges=new CharRanges();");
           } //end Extender
           if (local.equals("PubidChar")) {
             output.println("this.setPubidChar(ranges);");
             output.println("");
             output.println("ranges=new CharRanges();");
           } //end PubidChar
        }//end namespace


}//end endElement

   
}