package com.simonstl.gorille;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import java.io.IOException;
import java.util.*;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001-2. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>This class loads the information stored in a chars document for use in character verification.  It's based on Ethan Cerami's Xerces Quickstart, available at:</p>
* <p><a href="http://ecerami.com/xerces/" target="_top">http://ecerami.com/xerces/</a></p>
*
* <p>Version 0.04 shifts from attributes as content-holders to elements.</p>
* <p>Version 0.03 shifts from chars to ints to support surrogate pairs and values >0x10000.</p>
* <p>Version 0.02 adds support for the PubidChar production.</p>
* <p>Version 0.01 is based on the RulesLoader class from the <a href="http://simonstl.com/projects/fragment">com.simonstl.fragment</a> package.</p>
*
* @version 0.04 16 October 2002
* @author Simon St.Laurent
*/

public class CharRulesLoader extends DefaultHandler {
    
    public CharRules rules;   //set of rules
    protected CharRanges ranges;
    protected String start=null;
    protected String end=null;
    protected StringBuffer collector=null;
    protected boolean startEnd=false;
    protected boolean listening=false;

    // Constructor
    public CharRulesLoader (String xmlFile, XMLReader parser) {
        
        //  Set Content Handler
        parser.setContentHandler (this);

        // Create a rules container
        rules=new CharRules();

        // Create an initial range container
        ranges=new CharRanges();

 
        //  Parse the Document      
        try {
            parser.parse(xmlFile);
        } catch (SAXException e) {
            System.err.println (e);
        } catch (IOException e) {
            System.err.println (e);
        }
    }
    
    //  Start Element Event Handler
public void startElement (String uri, String local,
        String qName, Attributes atts)  {

       
        if (uri.equals("http://simonstl.com/ns/gorille/")) {
//System.out.println("Starting: " + qName);
        if ( (local.equals("hex")) || (local.equals("chars")) ){
            start=null;
            end=null;
            collector=new StringBuffer();
            startEnd=false;
            listening=true;
        }

       if (local.equals("start")){
            startEnd=true;
       }


       }//end namespace
}//end startElement

public void characters(char[] ch, int offset, int len) throws SAXException{
//System.out.println(listening);
   if (listening) {
      collector.append(ch, offset, len);
   }
}


public void endElement (String uri, String local,
        String qName)  {
        if (uri.equals("http://simonstl.com/ns/gorille/")) {
//System.out.println("Ending: " + qName);
          if (local.equals("start")){
            start=collector.toString();
            collector=new StringBuffer();
          }
          
          if (local.equals("end")){
            startEnd=true;
            end=collector.toString();
            collector=new StringBuffer();
          }
          
          if ( (local.equals("hex")) || (local.equals("chars")) ){
              if (startEnd==false) {
                 start=collector.toString();
              }
              startEnd=false;
              listening=false;
        }
           if (local.equals("hex")) {
             int startInt=Integer.decode(start).intValue();
//System.out.println("Starting at "+startInt);

             CharRange range;

             if (end==null) {
               range=new CharRange(startInt);
               ranges.addRange(range);
             } else {
             int endInt=Integer.decode(end).intValue();
//System.out.println("Ending at "+endInt);
               range=new CharRange(startInt, endInt);
               ranges.addRange(range);
             }
           } //end hex
           
           
           if (local.equals("chars")) {
//System.out.println("Char is "+start);           
             int startInt=start.charAt(0);
//System.out.println("Char is "+startInt);

             CharRange range;

             if (end==null) {
               range=new CharRange(startInt);
               ranges.addRange(range);
             } else {
             int endInt=end.charAt(0);
;
//System.out.println("Char is "+endChar);
               range=new CharRange(startInt, endInt);
               ranges.addRange(range);
             }
           } //end chars
           
           
           if (local.equals("Char")) {
             rules.setChar(ranges);
             ranges=new CharRanges();
           } //end Char
           if (local.equals("NameStartChar")) {
             rules.setNameStartChar(ranges);
             ranges=new CharRanges();
           } //end Char
           if (local.equals("AddlNameChar")) {
             rules.setAddlNameChar(ranges);
             ranges=new CharRanges();
           } //end Char
           if (local.equals("Space")) {
             rules.setSpace(ranges);
             ranges=new CharRanges();
           } //end Space
           if (local.equals("BaseChar")) {
             rules.setBaseChar(ranges);
             ranges=new CharRanges();
           } //end BaseChar
           if (local.equals("Ideographic")) {
             rules.setIdeographic(ranges);
             ranges=new CharRanges();
           } //end Ideographic
           if (local.equals("CombiningChar")) {
             rules.setCombiningChar(ranges);
             ranges=new CharRanges();
           } //end CombiningChar
           if (local.equals("Digit")) {
             rules.setDigit(ranges);
             ranges=new CharRanges();
           } //end Digit
           if (local.equals("Extender")) {
             rules.setExtender(ranges);
             ranges=new CharRanges();
           } //end Extender
           if (local.equals("PubidChar")) {
             rules.setPubidChar(ranges);
             ranges=new CharRanges();
           } //end PubidChar
        }//end namespace


}//end endElement

   
}