package com.simonstl.gorille;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

public class CharTester {

/**
* <p>This class tests {@link com.simonstl.gorille.TestCharRules TestCharRules}.
* It takes two arguments - a rules file and a target file.
* It loads the rules and feeds them to the filter
* so that testers can see the results on the command line.</p>
*
*
* <p>Version 0.03 adds a test for character classes created by {@link com.simonstl.gorille.CharRulesGen CharRulesGen} as well as command-line options for creating them.</p>
* <p>Version 0.02 adds a fixed test for characters beyond plane 0.  It should only work when characters >10000 are permissible.</p>
* <p>Version 0.01 is based on the FilterTester class from the <a href="http://simonstl.com/projects/fragment">com.simonstl.fragment</a> package.</p>
*
* @version 0.03 18 January 2002
* @author Simon St.Laurent
*/

/**
printHelp() lists options if the user doesn't specify any files.
*/

public static void printHelp() {
    System.out.println("CharTester requires two arguments - a rules file and a test file.");
    System.out.println("");
    System.out.println("Options may appear before those arguments:");
    System.out.println("-p [parser class name] - specifies a SAX2 parser for use with this tool");
    System.out.println("-r  - parse and display the rules only");
    System.out.println("-s  - try the SAX filter on the document");
    System.out.println("-g  - generate code");
    System.out.println("More information is available at http://simonstl.com/projects/gorille/");
    System.out.println("");
/*
    System.out.println("");
    System.out.println("");
    System.out.println("");
    System.out.println("");
*/
}

/**
FilterTester's main method requires two arguments - a rules file and a target XML file.  A rules file may also be specified.

Options may appear before those arguments:<br />
-p [parser class name] - specifies a SAX2 parser for use with this tool<br />
-r  - parse and display the rules only<br />
-s  - try the SAX filter on the document
*/
/*
  // Main Method
  public static void main (String[] args) throws Exception {

    int numArgs=args.length;
    boolean printRules=false;
    boolean saxFilter=false;
    boolean generateCode=false;
    boolean testGenerated=false;
    String rulesFile="";
    String targetFile="";

//deal with System properties
   if ((System.getProperty("org.xml.sax.driver"))==null) {
      System.setProperty("org.xml.sax.driver","org.apache.xerces.parsers.SAXParser");
   }


   String parserName=System.getProperty("org.xml.sax.driver");

    if (numArgs<1) {
      printHelp();
    } else {
//process arguments
   if (numArgs>1) {
       rulesFile=args[numArgs-2];
       targetFile=args[numArgs-1];
   }
  for (int i=0; i<numArgs; i++) {
   if (args[i].equals ("-p")) {
       parserName=args[i+1];
       i++;
   }
   if (args[i].equals ("-r")) {
       printRules=true;
       rulesFile=args[i+1];
       i++;
   }
   if (args[i].equals ("-s")) {
       saxFilter=true;
       rulesFile=args[i+1];
       i++;
   }
   if (args[i].equals ("-g")) {
       generateCode=true;
       rulesFile=args[i+1];
       i++;
   }
   if (args[i].equals ("-t")) {
       //this option only tests a class named "GeneratedRules"
       //by printing its contents
       testGenerated=true;
       break;
   }

  }

//  Create a SAX Parser
//XMLReader rulesParser = XMLReaderFactory.createXMLReader(parserName);

     if (generateCode) {
//        CharRulesGen thisRulesGen = new CharRulesGen (rulesFile, rulesParser);

       System.exit(0);
     }
     if (testGenerated) {
        CharRules rules=new GeneratedRules ();
        rules.print();
        System.exit(0);
     }

        CharRulesLoader thisRulesLoader = new CharRulesLoader (rulesFile, rulesParser);
        CharRules rules=thisRulesLoader.rules;

//        XMLReader parser = XMLReaderFactory.createXMLReader(parserName);
        if (printRules) {
          rules.print();
        } else  {
          if (saxFilter) {
//            CharRulesFilter filter=new CharRulesFilter(parser);
//            LocatorImpl locator=new LocatorImpl();
//            filter.setRules(rules);
//            filter.setVerbose(true);
//            filter.setDocumentLocator(locator);
//            filter.parse(new InputSource(new FileReader(targetFile)));

          } else {
//
//            TestCharRules tester=new TestCharRules(parser);
//            tester.setRules(rules);
//            tester.parse(new InputSource(new FileReader(targetFile)));

          //fixed tests for Planes beyond 0

         StringBuffer testString=new StringBuffer();

         testString=new StringBuffer();
         char firstSurrogate=0xD800;
         testString.append(firstSurrogate);
         char secondSurrogate=0xDC20;
         testString.append(secondSurrogate);
         char thirdSurrogate=0xD801;
         testString.append(thirdSurrogate);
         char fourthSurrogate=0xDC21;
         testString.append(fourthSurrogate);

         String testStringer=testString.toString();

         boolean result=rules.isName(testStringer);
//System.out.println("Test string:"+testStringer);
         System.out.println("Result for correctly formed surrogate pairs isName:"+result);

         testString=new StringBuffer();
         firstSurrogate=0xDC20;
         testString.append(firstSurrogate);
         secondSurrogate=0xD800;
         testString.append(secondSurrogate);
         thirdSurrogate=0xDC20;
         testString.append(thirdSurrogate);
         fourthSurrogate=0xDC20;
         testString.append(fourthSurrogate);

         testStringer=testString.toString();

         result=rules.isName(testStringer);

         System.out.println("Result for poorly formed surrogate pairs isName:"+result);


         testString=new StringBuffer();

         testString=new StringBuffer();
         firstSurrogate=0xD800;
         testString.append(firstSurrogate);
         secondSurrogate=0x0020;
         testString.append(secondSurrogate);
         thirdSurrogate=0xD800;
         testString.append(thirdSurrogate);
         fourthSurrogate=0xDC20;
         testString.append(fourthSurrogate);

         testStringer=testString.toString();

         result=rules.isName(testStringer);

         System.out.println("Result for poorly formed surrogate pairs isName:"+result);


          }
        }
    }
  }
*/
}