package com.simonstl.gorille;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License.

The Original Code is available at  http://simonstl.com/projects/fragment/original.

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved.

Contributor(s):
*/

import com.simonstl.common.*;

import java.util.Hashtable;

/**
 * Created by IntelliJ IDEA.
 * User: simon
 * Date: Mar 8, 2003
 * Time: 10:46:01 AM
 */
public class Context implements ContextI {

    protected String origin=null;
    protected String version="1.0";
    protected String encoding="UTF-8";
    protected String standalone="no";
    protected boolean xmlDecl=false;
    protected NSContext namespaces=new NSContext();
    protected StackableComponentI current=new StackableComponent(NodeType.DOCUMENT);
    //Entities from application
    protected Hashtable entities=new Hashtable();

    public Context() {
        reset();
    }

    /**
     * Accepts a string identifying the origin of the document.  Most likely a URI stating where it came from.
     */
    public void setOrigin(String origin) {
        this.origin=origin;
    }

    /**
     * Returns a string identifying the origin of the document.  Most likely a URI stating where it came from.
     */
    public String getOrigin() {
        return origin;
    }

    /**
        *   Takes a complete XML declaration for processing
        * @param declaration    the complete XML declaration, from <? to ?>
        */
       public boolean setXMLDeclaration(String declaration) {
           int state=0;
           boolean quoteType=false; //true=double
           boolean quoteOpened=false;
           final int VERSION=0;
           final int VERSION_VALUE=1;
           final int ENCODING=2;
           final int ENCODING_VALUE=3;
           final int STANDALONE=4;
           final int STANDALONE_VALUE=5;


           if (declaration.length()==7) {return true;}

           StringBuffer cleaner=new StringBuffer();
           StringBuffer current=new StringBuffer();

           for (int i=5; i<(declaration.length()-2);i++){
               char currentChar=declaration.charAt(i);
               if (isSpace(currentChar)){
                   //skip, possible checking here later
               } else {
                   cleaner.append(currentChar);
               }
           }


           String condDeclaration=cleaner.toString();

           if (condDeclaration.length()==0) {return true;}


           char initial=condDeclaration.charAt(0);
           if (initial=='e') {state=ENCODING;}

           boolean putInBuffer=true;

           for (int i=0;i<condDeclaration.length();i++){
               char currentChar=condDeclaration.charAt(i);
               if (state==VERSION){
                   if (currentChar=='='){
                       //complete, process version
                       String name=current.toString();
                       if (!(name.equals("version"))) {return false;}
                       state=VERSION_VALUE;
                       current=new StringBuffer();
                       putInBuffer=false;
                   }
               }

               if (state==VERSION_VALUE){
                   if (currentChar=='"'){
                       putInBuffer=false;
                       if (quoteOpened){
                           //closing quote
                           if (quoteType){
                               String value=current.toString();
                               current=new StringBuffer();
                               this.setVersion(value);
                               quoteOpened=false;
                               state=ENCODING;
                           } else {

                               return false;
                           }
                       } else {
                           //initial quote
                           quoteOpened=true;
                           quoteType=true;
                       }
                   } else if (currentChar=='\'') {
                       //single quote
                       putInBuffer=false;
                       if (quoteOpened){
                           //closing quote
                           if (!quoteType){
                               String value=current.toString();
                               current=new StringBuffer();
                               this.setVersion(value);
                               quoteOpened=false;
                               state=ENCODING;
                           } else {

                               return false;
                           }
                       } else {
                           //initial quote
                           quoteOpened=true;
                           quoteType=false;
                       }
                   } else {
                       //nothing, just collect
                   }

               }


               if (state==ENCODING){
                   if (currentChar=='='){
                       //complete, process version
                       String name=current.toString();
                       if (!(name.equals("encoding"))) {return false;}
                       state=ENCODING_VALUE;
                       current=new StringBuffer();
                       putInBuffer=false;
                   }
               }

               if (state==ENCODING_VALUE){
                   if (currentChar=='"'){
                       putInBuffer=false;
                       if (quoteOpened){
                           //closing quote
                           if (quoteType){
                               String value=current.toString();
                               current=new StringBuffer();
                               this.setEncoding(value);
                               quoteOpened=false;
                               state=STANDALONE;
                           } else {

                               return false;
                           }
                       } else {
                           //initial quote
                           quoteOpened=true;
                           quoteType=true;
                       }
                   } else if (currentChar=='\'') {
                       //single quote
                       putInBuffer=false;
                       if (quoteOpened){
                           //closing quote
                           if (!quoteType){
                               String value=current.toString();
                               current=new StringBuffer();
                               this.setEncoding(value);
                               quoteOpened=false;
                               state=STANDALONE;
                           } else {

                               return false;
                           }
                       } else {
                           //initial quote
                           quoteOpened=true;
                           quoteType=false;
                       }
                   } else {
                       //nothing, just collect
                   }

               }


               if (state==STANDALONE){
                   if (currentChar=='='){
                       //complete, process version
                       String name=current.toString();
                       if (!(name.equals("standalone"))) {return false;}
                       state=STANDALONE_VALUE;
                       current=new StringBuffer();
                       putInBuffer=false;
                   }
               }

               if (state==STANDALONE_VALUE){
                   if (currentChar=='"'){
                       putInBuffer=false;
                       if (quoteOpened){
                           //closing quote
                           if (quoteType){
                               String value=current.toString();
                               current=new StringBuffer();
                               this.setStandalone(value);
                               quoteOpened=false;
                               //
                           } else {

                               return false;
                           }
                       } else {
                           //initial quote
                           quoteOpened=true;
                           quoteType=true;
                       }
                   } else if (currentChar=='\'') {
                       //single quote
                       putInBuffer=false;
                       if (quoteOpened){
                           //closing quote
                           if (!quoteType){
                               String value=current.toString();
                               current=new StringBuffer();
                               this.setStandalone(value);
                               quoteOpened=false;
                               //
                           } else {

                               return false;
                           }
                       } else {
                           //initial quote
                           quoteOpened=true;
                           quoteType=false;
                       }
                   } else {
                       //nothing, just collect
                   }

               }


               if (putInBuffer) {current.append(currentChar);}
               putInBuffer=true;
           }

           this.setExplicitXMLDecl(true);

           return true;
       }


    /**
     * Sets the version of XML used by the document, retrieved from
     * the XML declaration.  Should default to "1.0".
     *
     * @param version
     */
    public void setVersion(String version) {
        this.version=version;
    }

    /**
     * Provides access to the version information from the XML
     * declaration.
     *
     * @return the version of XML used by the document
     */
    public String getVersion() {
        return version;
    }

    /**
     * Sets the character encoding of XML used by the document,
     * retrieved from
     * the XML declaration.  Should default to "1.0".
     *
     * @param encoding
     */
    public void setEncoding(String encoding) {
        this.encoding=encoding;
    }

    /**
     * Provides access to the encoding information from the XML
     * declaration.
     *
     * @return the character encoding specified by the document
     */
    public String getEncoding() {
        return encoding;
    }

    /**
     * Sets the value of the standalone declaration, retrieved from
     * the XML declaration.  Should default to "no". "yes" is the other option.
     *
     * @param standalone
     */

    public void setStandalone(String standalone) {
        this.standalone=standalone;
    }

    /**
     * Provides access to the standalone information from the XML
     * declaration.
     *
     * @return whether the document declares itself standalone
     */
    public String getStandalone() {
        return standalone;
    }

    /**
     * Identifies whether the document contained an explicit XML declaration.
     *
     * @param happened - true if the document had a decl, false if not
     */
    public void setExplicitXMLDecl(boolean happened) {
        xmlDecl=happened;
    }

    /**
     * Identifies whether or not the document included an XML
     * declaration.
     *
     * @return the version of XML used by the document
     */
    public boolean getExplicitXMLDecl() {
        return xmlDecl;
    }

    /**
     * Returns the component which is the parent of the current node.
     *
     */

    public StackableComponentI getParent() {
        return current.getParent();
    }

    /**
     * Returns the current node.
     *
     */

    public StackableComponentI getCurrent() {

        return current;
    }


    /**
     * Adds a new node to the stack and adjusts context.
     *
     */

    public void startChild(StackableComponentI child) {
        child.setParent(current);
        current.addChild(child);
        current=child;
        pushLevel();
    }

    /**
     * Ends a node and adjusts context.
     *
     */

    public void endChild (){
        current=current.getParent();
        popLevel();

    }

    /**
     * Adds a childless node.
     *
     */

    public void addNode (StackableComponentI child){
        startChild(child);
        endChild();
    }

    /**
     * Accepts a namespace declaration mapping a prefix to a URI.
     */
    public void declarePrefix(String prefix, String URI) {
        namespaces.declarePrefix(prefix, URI);
    }

    /**
     * Returns the URI which currently maps to a particular prefix.
     */
    public String getUri(String prefix) {
        String returnVal = namespaces.getUri(prefix);
        if (returnVal==null){returnVal="";}
        return returnVal;
    }

    /**
     * Returns a prefix which currently maps to a particular URI.  There can be
     * multiple prefixes for a particular URI, and this method only promises to return
     * one of them.
     */
    public String getPrefix(String URI) {
        String returnVal = namespaces.getPrefix(URI);
        if (returnVal==null){returnVal="";}
        return returnVal;
    }

    public void pushLevel() {

        namespaces.pushLevel();
    }

    public void popLevel()     {

        namespaces.popLevel();
    }

     public void setEntity(String name, String value) {
        entities.put(name, value);
    }

    public String resolveEntity(String entName){
        String entValue=null;
        int length=entName.length();
        if (entName.charAt(0)=='#'){
          int value;
            if (entName.charAt(1)=='x'){
                //hex char ref
                char[] chars=new char[length];
                chars[0]='0';
                entName.getChars(1,length,chars,1);
                String val=new String(chars);

                value=Integer.decode(val).intValue();

            }  else {
                //dec char ref
                char[] chars=new char[length-1];

                entName.getChars(1,length,chars,0);
                String val=new String(chars);

                value=Integer.decode(val).intValue();

            }
            if (value<65535){
                //regular char, one char output
                entValue=String.valueOf((char)value);

            } else {
                //surrogate pair handling
                char[] myChars=new char[2];
                int s=value-65546;
                int high=(s / 1024) + 55296;
                int low=(s % 1024) + 56320;
                myChars[0]=((char)high);
                myChars[1]=(char)low;
                entValue=new String(myChars);
            }
        }   else {
            entValue=(String) entities.get(entName);
        }
        return entValue;
    }

    /**
     *  Reports whether the character c is whitespace or not.
     * @param c a character to test
     * @return  true if it's whitespace, false if it's not
     */
    public boolean isSpace(char c){
        boolean returnVal=false;

        if ((c==' ') || (c==0x9) || (c==0xA) || (c==0xD)) {returnVal=true;}

        return returnVal;
    }

    public void reset() {
         origin=null;
         namespaces.reset();
         current=new StackableComponent(NodeType.DOCUMENT);
         entities.clear();
        entities.put("lt", "<");
        entities.put("gt", ">");
        entities.put("amp", "&");
        entities.put("apos", "'");
        entities.put("quot", "\"");
    }

     public static void main(String[] args) throws Exception{

         Context context=new Context();

         String test="#x4D";
         System.out.println("&"+test+";");
         System.out.println(context.resolveEntity(test));

         test="#67";
         System.out.println("&"+test+";");
         System.out.println(context.resolveEntity(test));

         test="amp";
         System.out.println("&"+test+";");
         System.out.println(context.resolveEntity(test));

         context.setEntity("monkey","starlight");

         test="monkey";
         System.out.println("&"+test+";");
         System.out.println(context.resolveEntity(test));
     }
}
