package com.simonstl.gorille;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License.

The Original Code is available at  http://simonstl.com/projects/fragment/original.

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2002. All Rights Reserved.

Contributor(s):
*/

import com.simonstl.common.*;


/**
 * Represents the context for the current event.  Events are added to the context
 * _after_ they have been through the filtering process.
 * 
 * 
 *
 * @author  simonstl
 */

public interface ContextI extends NamespaceStackI{

//origin
    /**
     * Accepts a string identifying the origin of the document.  Most likely a URI stating where it came from.
     */
    public void setOrigin(String origin);


    /**
     * Returns a string identifying the origin of the document.  Most likely a URI stating where it came from.
     */
    public String getOrigin();

    /**
     *   Takes a complete XML declaration for processing; returns false if there's an error.
     * @param declaration    the complete XML declaration, from <? to ?>
     */
    public boolean setXMLDeclaration(String declaration);

    /**
     * Sets the version of XML used by the document, retrieved from
     * the XML declaration.  Should default to "1.0".
     *
     * @param version
     */
    public void setVersion(String version);
    /**
     * Provides access to the version information from the XML
     * declaration.
     *
     * @return the version of XML used by the document
     */
    public String getVersion();
    /**
     * Sets the character encoding of XML used by the document,
     * retrieved from
     * the XML declaration.  Should default to "1.0".
     *
     * @param encoding
     */
    public void setEncoding(String encoding);

    /**
     * Provides access to the encoding information from the XML
     * declaration.
     *
     * @return the character encoding specified by the document
     */
    public String getEncoding();

    /**
     * Sets the value of the standalone declaration, retrieved from
     * the XML declaration.  Should default to "no". "yes" is the other option.
     *
     * @param standalone
     */

    public void setStandalone(String standalone);

    /**
     * Provides access to the standalone information from the XML
     * declaration.
     *
     * @return whether the document declares itself standalone
     */
    public String getStandalone();
    /**
     * Identifies whether the document contained an explicit XML declaration.
     *
     * @param happened - true if the document had a decl, false if not
     */
    public void setExplicitXMLDecl(boolean happened);
     /**
     * Identifies whether or not the document included an XML
     * declaration.
     *
     * @return the version of XML used by the document
     */
    public boolean getExplicitXMLDecl();

//parents/prior siblings/attributes

    /**
     * Returns the component which is the parent of the current node.
     *
     */

    public StackableComponentI getParent();

    /**
     * Returns the current node.
     *
     */

    public StackableComponentI getCurrent();

    /**
     * Adds a new node to the stack and adjusts context.
     *
     */

    public void startChild (StackableComponentI child);

    /**
     * End a node and adjusts context.
     *
     */

    public void endChild ();

   /**
     * Adds a childless node.
     *
     */
    public void addNode (StackableComponentI child);
    /**
     *  Establishes an entity name-value relationship. The value must
     * be provided directly and not by reference.
     * @param name      name of the entity
     * @param value     value of the entity
     */
    public void setEntity(String name, String value);

    /**
     *  Resolves an entity, hexRef, or decRef.
     * @param entName  the name of the entity or ref, minus the & and ;
     * @return a string containing either the entity or hex/dec processing result
     */
    public String resolveEntity(String entName);
    /**
     *  Reports whether the character c is whitespace or not.
     * @param c a character to test
     * @return  true if it's whitespace, false if it's not
     */
    public boolean isSpace(char c);
//stacking
    public void pushLevel();//only used when elements start
    public void popLevel();//only used when elements end

//clear everything
    public void reset();

}