package com.simonstl.gorille;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License.

The Original Code is available at  http://simonstl.com/projects/fragment/original.

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved.

Contributor(s):
*/
import java.io.InputStream;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.Reader;


/**
 * Created by IntelliJ IDEA.
 * User: simon
 * Date: Mar 8, 2003
 * Time: 6:48:04 PM
 * To change this template use Options | File Templates.
 *
 * start/end element nesting verification
 * add non-element/att nodes to context.
 * reconsider interface approach
 *
 */
public class RipperTest {


    public static void printHelp(){
        System.out.println("Ripper Test requires at least one argument, a test file. After the URL, you can use on of the following options:");
        System.out.println("-P reporting as parts, verbose");
        System.out.println("-p reporting as parts, quieter");
        System.out.println("-L use LoudContext reporting.");
        System.out.println("You can also use the -r argument to parse a resource on the CLASSPATH rather than a file.");

    }

    public void test(String[] args) throws Exception {

        int numArgs=args.length;
        boolean parts=false;
        boolean verboseParts=false;
        boolean fromResource=false;
        boolean loudContext=false;

        String targetFile=null;

        targetFile=args[0];

        for (int i=1; i<numArgs; i++){
            String arg=args[i];
            if (arg.equals("-P")) {
                parts=true;
                verboseParts=true;
            }
            if (arg.equals("-p")) {
                parts=true;
            }
            if (arg.equals("-r")) {
                fromResource=true;
            }
            if (arg.equals("-L")) {
                loudContext=true;
            }
        }

        Ripper test=new Ripper();

        if (loudContext) {
            test.setContext(new LoudContext());
        }



        Reader r;

        if (fromResource){
            InputStream is=this.getClass().getResourceAsStream(targetFile);
            r=new InputStreamReader(is);
        } else {
            //need to fix
            r=(new FileReader(targetFile));
        }




        if (parts){

            DocProcReporter handler=new DocProcReporter();

            handler.setVerbose(verboseParts);

            test.setHandler(handler);
            test.parse(r);
        } else {


            DocProcCollector handler=new DocProcCollector();

            test.setHandler(handler);

            test.parse(r);
            handler.print();

        }


    }

    public static void main(String[] args) throws Exception{
        if (args.length<1){
            printHelp();
        } else {

            RipperTest tester=new RipperTest();
            tester.test(args);

        }
    }

}
