package com.simonstl.gorille;

import org.xml.sax.*;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.XMLFilter;
import org.xml.sax.helpers.XMLFilterImpl;
import java.util.*;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 

Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
*
* <p>A test for the XML character rules.  See <code>test.xml</code> for a sample of the test file.</p>
*
*<p>Version 0.03 adds support for Chars (characters) testing.</p>
*<p>Version 0.02 adds support for Pubid (PUBLIC identifier) testing.</p>
*
* @version 0.02  3 January 2002
* @author Simon St.Laurent
*/

public class TestCharRules extends XMLFilterImpl { 

    private CharRules rules=null;   //set of rules
    String charsExpectedResult=null;
    boolean processChars=false;

/**
* An empty constructor that requires the use of setParent before starting
* I don't think this works.
*/
public TestCharRules () {

 } 

/**
* A constructor that takes the parser which will feed it SAX events
*/

public TestCharRules (XMLReader parent) {
    super(parent); 
} 

/**
* sets up the rules, read in from a config file by {@link com.simonstl.gorille.CharRulesLoader CharRulesLoader} or
* otherwise concocted in Java using the {@link com.simonstl.gorille.CharRules CharRules} class.
*
* @param newRules  the set of rules the class will apply to content
*/
public void setRules (CharRules newRules) {
  rules=newRules;
//rules.print();
}

/**
* in case you ever need to get the rules out of the tester 
*/
public CharRules getRules () {
  return rules;
}


public void startElement (String uri, String local, String qName, Attributes atts) throws SAXException {

 if (uri.equals("http://simonstl.com/ns/gorille/test/")) {
    if (local.equals("test")) {
      String target=atts.getValue("target");
      System.out.println("Results intended for: " + target);
    }
    if (local.equals("name")) {
      String expectedResult=atts.getValue("result");
      String value=atts.getValue("value");
      boolean result=rules.isName(value);

      System.out.println("Expected result for NAME "+value+": " + expectedResult);
      System.out.println("Real result for NAME "+value+": " + result);
    }
    if (local.equals("qname")) {
      String expectedResult=atts.getValue("result");
      String value=atts.getValue("value");
      boolean result=rules.isQName(value);

      System.out.println("Expected result for QNAME "+value+": " + expectedResult);
      System.out.println("Real result for QNAME "+value+": " + result);
    }
    if (local.equals("names")) {
      String expectedResult=atts.getValue("result");
      String value=atts.getValue("value");
      boolean result=rules.isNames(value);

      System.out.println("Expected result for NAMES "+value+": " + expectedResult);
      System.out.println("Real result for NAMES "+value+": " + result);
    }
    if (local.equals("nmtoken")) {
      String expectedResult=atts.getValue("result");
      String value=atts.getValue("value");
      boolean result=rules.isNMTOKEN(value);

      System.out.println("Expected result for NMTOKEN "+value+": " + expectedResult);
      System.out.println("Real result for NMTOKEN "+value+": " + result);
    }
    if (local.equals("nmtokens")) {
      String expectedResult=atts.getValue("result");
      String value=atts.getValue("value");
      boolean result=rules.isNMTOKENS(value);

      System.out.println("Expected result for NMTOKENS "+value+": " + expectedResult);
      System.out.println("Real result for NMTOKENS "+value+": " + result);
    }
    if (local.equals("pubid")) {
      String expectedResult=atts.getValue("result");
      String value=atts.getValue("value");
      boolean result=rules.isPubid(value);

      System.out.println("Expected result for Pubid "+value+": " + expectedResult);
      System.out.println("Real result for Pubid "+value+": " + result);
    }
    if (local.equals("chars")) {
      charsExpectedResult=atts.getValue("result");
      processChars=true;
    }


 }

  super.startElement(uri, local, qName, atts); 
}

public void characters (char ch[], int start, int length) throws SAXException {
   if (processChars) {

//System.out.println("Length: "+length);
      boolean result=rules.isChars(ch, start, length);
      String value=new String(ch, start, length);
      System.out.println("Expected result for Chars "+value+": " + charsExpectedResult);
      System.out.println("Real result for Chars "+value+": " + result);
    }
   super.characters (ch, start, length);
}

public void endElement (String uri, String local, String qName) throws SAXException {

 if (uri.equals("http://simonstl.com/ns/gorille/test/")) {
    if (local.equals("chars")) {
      processChars=false;
    }


 }

  super.endElement(uri, local, qName); 
}

} 
